@@ -1,4 +1,4 @@
-{-# LANGUAGE DataKinds, DeriveDataTypeable, DeriveGeneric, FlexibleContexts, FlexibleInstances, GeneralizedNewtypeDeriving, MultiParamTypeClasses, RecordWildCards, ScopedTypeVariables, StandaloneDeriving, TemplateHaskell, TypeOperators, TypeFamilies, TypeSynonymInstances, UndecidableInstances, OverloadedStrings #-}
+{-# LANGUAGE CPP, DataKinds, DeriveDataTypeable, DeriveGeneric, FlexibleContexts, FlexibleInstances, GeneralizedNewtypeDeriving, MultiParamTypeClasses, RecordWildCards, ScopedTypeVariables, StandaloneDeriving, TemplateHaskell, TypeOperators, TypeFamilies, TypeSynonymInstances, UndecidableInstances, OverloadedStrings #-}
 {-
 
 A user is uniquely identified by their 'UserId'. A user can have one
@@ -60,6 +60,7 @@ module Happstack.Authenticate.Core
     , IxUser
     , SharedSecret(..)
     , unSharedSecret
+    , SimpleAddress(..)
     , genSharedSecret
     , genSharedSecretDevURandom
     , genSharedSecretSysRandom
@@ -110,6 +111,9 @@ module Happstack.Authenticate.Core
     , AuthenticateURL(..)
     , rAuthenticationMethods
     , rControllers
+    , systemFromAddress
+    , systemReplyToAddress
+    , systemSendmailPath
     , authenticateURL
     , nestAuthenticationMethod
     ) where
@@ -135,8 +139,8 @@ import Data.Default                    (def)
 import Data.Map                        (Map)
 import qualified Data.Map              as Map
 import Data.Maybe                      (fromMaybe, maybeToList)
-import Data.Monoid                     ((<>), mconcat)
-import Data.SafeCopy                   (SafeCopy, base, deriveSafeCopy)
+import Data.Monoid                     ((<>), mconcat, mempty)
+import Data.SafeCopy                   (SafeCopy, Migrate(..), base, deriveSafeCopy, extension)
 import Data.IxSet.Typed
 import qualified Data.IxSet.Typed      as IxSet
 import           Data.Set              (Set)
@@ -156,12 +160,24 @@ import System.IO                       (IOMode(ReadMode), withFile)
 import System.Random                   (randomRIO)
 import Text.Boomerang.TH               (makeBoomerangs)
 import Text.Shakespeare.I18N           (RenderMessage(renderMessage), mkMessageFor)
-import Web.JWT                         (Algorithm(HS256), JWT, VerifiedJWT, JWTClaimsSet(..), encodeSigned, claims, decode, decodeAndVerifySignature, secondsSinceEpoch, intDate, secret, verify)
+import Web.JWT                         (Algorithm(HS256), JWT, VerifiedJWT, JWTClaimsSet(..), encodeSigned, claims, decode, decodeAndVerifySignature, secondsSinceEpoch, intDate, verify)
+#if MIN_VERSION_jwt(0,8,0)
+import Web.JWT                         (ClaimsMap(..), hmacSecret)
+#else
+import Web.JWT                         (secret)
+#endif
+
 import Web.Routes                      (RouteT, PathInfo(..), nestURL)
 import Web.Routes.Boomerang
 import Web.Routes.Happstack            ()
 import Web.Routes.TH                   (derivePathInfo)
 
+#if MIN_VERSION_jwt(0,8,0)
+#else
+unClaimsMap = id
+#endif
+
+
 -- | when creating JSON field names, drop the first character. Since
 -- we are using lens, the leading character should always be _.
 jsonOptions :: Options
@@ -322,15 +338,30 @@ instance Indexable UserIxs User where
              (ixFun $ (:[]) . view username)
              (ixFun $ maybeToList . view email)
 
+------------------------------------------------------------------------------
+-- SimpleAddress
+------------------------------------------------------------------------------
+
+data SimpleAddress = SimpleAddress
+ { _saName :: Maybe Text
+ , _saEmail :: Email
+ }
+ deriving (Eq, Ord, Read, Show, Data, Typeable, Generic)
+deriveSafeCopy 0 'base ''SimpleAddress
+makeLenses ''SimpleAddress
+
 ------------------------------------------------------------------------------
 -- AuthenticateConfig
 ------------------------------------------------------------------------------
 
 -- | Various configuration options that apply to all authentication methods
 data AuthenticateConfig = AuthenticateConfig
-    { _isAuthAdmin        :: UserId -> IO Bool           -- ^ can user administrate the authentication system?
-    , _usernameAcceptable :: Username -> Maybe CoreError -- ^ enforce username policies, valid email, etc. 'Nothing' == ok, 'Just Text' == error message
-    , _requireEmail       :: Bool
+    { _isAuthAdmin          :: UserId -> IO Bool           -- ^ can user administrate the authentication system?
+    , _usernameAcceptable   :: Username -> Maybe CoreError -- ^ enforce username policies, valid email, etc. 'Nothing' == ok, 'Just Text' == error message
+    , _requireEmail         :: Bool                        -- ^ require use to supply an email address when creating an account
+    , _systemFromAddress    :: Maybe SimpleAddress         -- ^ From: line for emails sent by the server
+    , _systemReplyToAddress :: Maybe SimpleAddress         -- ^ Reply-To: line for emails sent by the server
+    , _systemSendmailPath   :: Maybe FilePath              -- ^ path to sendmail if it is not \/usr\/sbin\/sendmail
     }
     deriving (Typeable, Generic)
 makeLenses ''AuthenticateConfig
@@ -626,13 +657,29 @@ issueToken authenticateState authenticateConfig user =
   do ssecret <- getOrGenSharedSecret authenticateState (user ^. userId)
      admin   <- liftIO $ (authenticateConfig ^. isAuthAdmin) (user ^. userId)
      now <- liftIO getCurrentTime
-     let claims = def { exp = intDate $ utcTimeToPOSIXSeconds (addUTCTime (60*60*24*30) now)
-                      , unregisteredClaims =
+     let claims = JWTClaimsSet
+                   { iss = Nothing
+                   , sub = Nothing
+                   , aud = Nothing
+                   , exp = intDate $ utcTimeToPOSIXSeconds (addUTCTime (60*60*24*30) now)
+                   , nbf = Nothing
+                   , iat = Nothing
+                   , jti = Nothing
+                   , unregisteredClaims =
+#if MIN_VERSION_jwt(0,8,0)
+                         ClaimsMap $
+#endif
                            Map.fromList [ ("user"     , toJSON user)
                                         , ("authAdmin", toJSON admin)
                                         ]
-                      }
+                   }
+#if MIN_VERSION_jwt(0,10,0)
+     return $ encodeSigned (hmacSecret $ _unSharedSecret ssecret) mempty claims
+#elif MIN_VERSION_jwt(0,9,0)
+     return $ encodeSigned (hmacSecret $ _unSharedSecret ssecret) claims
+#else
      return $ encodeSigned HS256 (secret $ _unSharedSecret ssecret) claims
+#endif
 
 -- | decode and verify the `TokenText`. If successful, return the
 -- `Token` otherwise `Nothing`.
@@ -648,7 +695,7 @@ decodeAndVerifyToken authenticateState now token =
        Nothing -> return Nothing
        (Just unverified) ->
          -- check that token has user claim
-         case Map.lookup "user" (unregisteredClaims (claims unverified)) of
+         case Map.lookup "user" (unClaimsMap (unregisteredClaims (claims unverified))) of
            Nothing -> return Nothing
            (Just uv) ->
              -- decode user json value
@@ -661,7 +708,11 @@ decodeAndVerifyToken authenticateState now token =
                       Nothing -> return Nothing
                       (Just ssecret) ->
                         -- finally we can verify all the claims
+#if MIN_VERSION_jwt(0,8,0)
+                        case verify (hmacSecret (_unSharedSecret ssecret)) unverified of
+#else
                         case verify (secret (_unSharedSecret ssecret)) unverified of
+#endif
                           Nothing -> return Nothing
                           (Just verified) -> -- check expiration
                             case exp (claims verified) of
@@ -670,7 +721,7 @@ decodeAndVerifyToken authenticateState now token =
                               (Just exp') ->
                                 if (utcTimeToPOSIXSeconds now) > (secondsSinceEpoch exp')
                                 then return Nothing
-                                else case Map.lookup "authAdmin" (unregisteredClaims (claims verified)) of
+                                else case Map.lookup "authAdmin" (unClaimsMap (unregisteredClaims (claims verified))) of
                                        Nothing -> return (Just (Token u False, verified))
                                        (Just a) ->
                                            case fromJSON a of
