@@ -1,4 +1,4 @@
-{-# LANGUAGE DataKinds, DeriveDataTypeable, DeriveGeneric, FlexibleInstances, MultiParamTypeClasses, RecordWildCards, TemplateHaskell, TypeFamilies, TypeSynonymInstances, OverloadedStrings #-}
+{-# LANGUAGE CPP, DataKinds, DeriveDataTypeable, DeriveGeneric, FlexibleInstances, MultiParamTypeClasses, RecordWildCards, TemplateHaskell, TypeFamilies, TypeSynonymInstances, OverloadedStrings, StandaloneDeriving #-}
 module Happstack.Authenticate.Password.Core where
 
 import Control.Applicative ((<$>), optional)
@@ -20,8 +20,8 @@ import qualified Data.HashMap.Strict as HashMap
 import           Data.Map (Map)
 import qualified Data.Map as Map
 import Data.Maybe         (fromMaybe, fromJust)
-import Data.Monoid        ((<>))
-import Data.SafeCopy (SafeCopy, base, deriveSafeCopy)
+import Data.Monoid        ((<>), mempty)
+import Data.SafeCopy (SafeCopy, Migrate(..), base, extension, deriveSafeCopy)
 import           Data.Text (Text)
 import qualified Data.Text as Text
 import qualified Data.Text.IO as Text
@@ -30,21 +30,31 @@ import qualified Data.Text.Lazy     as LT
 import Data.Time.Clock.POSIX          (getPOSIXTime)
 import Data.UserId (UserId)
 import GHC.Generics (Generic)
-import Happstack.Authenticate.Core (AuthenticationHandler, AuthenticationMethod(..), AuthenticateState(..), AuthenticateConfig, usernameAcceptable, requireEmail, AuthenticateURL, CoreError(..), CreateUser(..), Email(..), unEmail, GetUserByUsername(..), HappstackAuthenticateI18N(..), SharedSecret(..), User(..), Username(..), GetSharedSecret(..), addTokenCookie, email, getToken, getOrGenSharedSecret, jsonOptions, userId, username, toJSONSuccess, toJSONResponse, toJSONError, tokenUser)
+import Happstack.Authenticate.Core (AuthenticationHandler, AuthenticationMethod(..), AuthenticateState(..), AuthenticateConfig, usernameAcceptable, requireEmail, AuthenticateURL, CoreError(..), CreateUser(..), Email(..), unEmail, GetUserByUsername(..), HappstackAuthenticateI18N(..), SharedSecret(..), SimpleAddress(..), User(..), Username(..), GetSharedSecret(..), addTokenCookie, email, getToken, getOrGenSharedSecret, jsonOptions, userId, username, systemFromAddress, systemReplyToAddress, systemSendmailPath, toJSONSuccess, toJSONResponse, toJSONError, tokenUser)
 import Happstack.Authenticate.Password.URL (AccountURL(..))
 import Happstack.Server
 import HSP.JMacro
 import Language.Javascript.JMacro
 import Network.HTTP.Types              (toQuery, renderQuery)
-import Network.Mail.Mime               (Address(..), Mail, simpleMail', renderMail', renderSendMail, sendmail)
+import Network.Mail.Mime               (Address(..), Mail(..), simpleMail', renderMail', renderSendMail, renderSendMailCustom, sendmail)
 import System.FilePath                 (combine)
 import qualified Text.Email.Validate   as Email
 import Text.Shakespeare.I18N           (RenderMessage(..), Lang, mkMessageFor)
 import qualified Web.JWT               as JWT
-import Web.JWT                         (Algorithm(HS256), JWT, VerifiedJWT, JWTClaimsSet(..), encodeSigned, claims, decode, decodeAndVerifySignature, intDate, secret, secondsSinceEpoch, verify)
+import Web.JWT                         (Algorithm(HS256), JWT, VerifiedJWT, JWTClaimsSet(..), encodeSigned, claims, decode, decodeAndVerifySignature, intDate, secondsSinceEpoch, verify)
+#if MIN_VERSION_jwt(0,8,0)
+import Web.JWT                         (ClaimsMap(..), hmacSecret)
+#else
+import Web.JWT                         (secret)
+#endif
 import Web.Routes
 import Web.Routes.TH
 
+#if MIN_VERSION_jwt(0,8,0)
+#else
+unClaimsMap = id
+#endif
+
 ------------------------------------------------------------------------------
 -- PasswordConfig
 ------------------------------------------------------------------------------
@@ -117,7 +127,7 @@ makeLenses ''PasswordState
 
 initialPasswordState :: PasswordState
 initialPasswordState = PasswordState
-    { _passwords = Map.empty
+    { _passwords      = Map.empty
     }
 
 ------------------------------------------------------------------------------
@@ -198,7 +208,7 @@ token :: (Happstack m) =>
       -> m Response
 token authenticateState authenticateConfig passwordState =
   do method POST
-     (Just (Body body)) <- takeRequestBody =<< askRq
+     ~(Just (Body body)) <- takeRequestBody =<< askRq
      case Aeson.decode body of
        Nothing   -> badRequest $ toJSONError (CoreError JSONDecodeFailed)
        (Just (UserPass username password)) ->
@@ -246,11 +256,11 @@ account :: (Happstack m) =>
         -> PasswordConfig
         -> Maybe (UserId, AccountURL)
         -> m (Either PasswordError UserId)
--- handle new account creation via POST to /account
+-- handle new account creation via POST to \/account
 -- FIXME: check that password and password confirmation match
 account authenticateState passwordState authenticateConfig passwordConfig Nothing =
   do method POST
-     (Just (Body body)) <- takeRequestBody =<< askRq
+     ~(Just (Body body)) <- takeRequestBody =<< askRq
      case Aeson.decode body of
        Nothing               -> badRequest (Left $ CoreError JSONDecodeFailed)
        (Just newAccount) ->
@@ -281,7 +291,7 @@ account authenticateState passwordState authenticateConfig passwordConfig Nothin
             (False, Nothing) -> Nothing
             (_, Just email) -> if Email.isValid (Text.encodeUtf8 (email ^. unEmail)) then Nothing else Just $ CoreError InvalidEmail
 
--- handle updates to /account/<userId>/*
+--  handle updates to '/account/<userId>/*'
 account authenticateState passwordState authenticateConfig passwordConfig (Just (uid, url)) =
   case url of
     Password ->
@@ -296,7 +306,7 @@ account authenticateState passwordState authenticateConfig passwordConfig (Just
               else do mBody <- takeRequestBody =<< askRq
                       case mBody of
                         Nothing     -> badRequest (Left $ CoreError JSONDecodeFailed)
-                        (Just (Body body)) ->
+                        ~(Just (Body body)) ->
                           case Aeson.decode body of
                             Nothing -> do -- liftIO $ print body
                                           badRequest (Left $ CoreError JSONDecodeFailed)
@@ -328,13 +338,14 @@ instance FromJSON RequestResetPasswordData where parseJSON = genericParseJSON js
 
 -- | request reset password
 passwordRequestReset :: (Happstack m) =>
-                        PasswordConfig
+                        AuthenticateConfig
+                     -> PasswordConfig
                      -> AcidState AuthenticateState
                      -> AcidState PasswordState
                      -> m (Either PasswordError Text)
-passwordRequestReset passwordConfig authenticateState passwordState =
+passwordRequestReset authenticateConfig passwordConfig authenticateState passwordState =
   do method POST
-     (Just (Body body)) <- takeRequestBody =<< askRq
+     ~(Just (Body body)) <- takeRequestBody =<< askRq
      case Aeson.decode body of
        Nothing   -> badRequest $ Left $ CoreError JSONDecodeFailed
        (Just (RequestResetPasswordData username)) ->
@@ -351,14 +362,15 @@ passwordRequestReset passwordConfig authenticateState passwordState =
                          (Right resetToken) ->
                            do let resetLink' = (passwordConfig ^. resetLink) <> (Text.decodeUtf8 $ renderQuery True $ toQuery [("reset_token"::Text, resetToken)])
                               liftIO $ Text.putStrLn resetLink' -- FIXME: don't print to stdout
-                              sendResetEmail toEm (Email ("no-reply@" <> (passwordConfig ^. domain))) resetLink'
+                              let from = fromMaybe (SimpleAddress Nothing (Email ("no-reply@" <> (passwordConfig ^. domain)))) (authenticateConfig ^. systemFromAddress)
+                              sendResetEmail (authenticateConfig ^. systemSendmailPath) toEm from (authenticateConfig ^. systemReplyToAddress) resetLink'
                               return (Right "password reset request email sent.") -- FIXME: I18N
 
 -- | issueResetToken
 issueResetToken :: (MonadIO m) =>
                    AcidState AuthenticateState
                 -> User
-                -> m (Either PasswordError JWT.JSON)
+                -> m (Either PasswordError Text)
 issueResetToken authenticateState user =
   case user ^. email of
     Nothing     -> return (Left NoEmailAddress)
@@ -366,22 +378,47 @@ issueResetToken authenticateState user =
       do ssecret <- getOrGenSharedSecret authenticateState (user ^. userId)
          -- FIXME: add expiration time
          now <- liftIO getPOSIXTime
-         let claims = JWT.def { unregisteredClaims = Map.singleton "reset-password" (toJSON user)
-                              , JWT.exp            = intDate $ now + 60
-                              }
+         let claims = JWT.JWTClaimsSet
+                        { JWT.iss = Nothing
+                        , JWT.sub = Nothing
+                        , JWT.aud = Nothing
+                        , JWT.exp = intDate $ now + 60
+                        , JWT.nbf = Nothing
+                        , JWT.iat = Nothing
+                        , JWT.jti = Nothing
+                        , JWT.unregisteredClaims =
+#if MIN_VERSION_jwt(0,8,0)
+                            JWT.ClaimsMap $
+#endif
+                               Map.singleton "reset-password" (toJSON user)
+                        }
+#if MIN_VERSION_jwt(0,10,0)
+         return $ Right $ encodeSigned (hmacSecret $ _unSharedSecret ssecret) mempty claims
+#elif MIN_VERSION_jwt(0,9,0)
+         return $ Right $ encodeSigned (hmacSecret $ _unSharedSecret ssecret) claims
+#else
          return $ Right $ encodeSigned HS256 (secret $ _unSharedSecret ssecret) claims
+#endif
 
 -- FIXME: I18N
 -- FIXME: call renderSendMail
 sendResetEmail :: (MonadIO m) =>
-                  Email
+                  Maybe FilePath
                -> Email
+               -> SimpleAddress
+               -> Maybe SimpleAddress
                -> Text
                -> m ()
-sendResetEmail (Email toEm) (Email fromEm) resetLink = liftIO $
-  do mailBS <- renderMail' $ simpleMail' (Address Nothing toEm)  (Address (Just "no-reply") fromEm) "Reset Password Request" (LT.fromStrict resetLink)
-     -- B.putStr mailBS
-     sendmail mailBS
+sendResetEmail mSendmailPath (Email toEm) (SimpleAddress fromNm (Email fromEm)) mReplyTo resetLink = liftIO $
+  do let mail = addReplyTo mReplyTo $ simpleMail' (Address Nothing toEm)  (Address fromNm fromEm) "Reset Password Request" (LT.fromStrict resetLink)
+     case mSendmailPath of
+       Nothing -> renderSendMail mail
+       (Just sendmailPath) -> renderSendMailCustom sendmailPath ["-t"] mail
+  where
+    addReplyTo :: Maybe SimpleAddress -> Mail -> Mail
+    addReplyTo Nothing m = m
+    addReplyTo (Just (SimpleAddress rplyToNm rplyToEm)) m =
+      let m' = m { mailHeaders = (mailHeaders m) } in m'
 
 -- | JSON record for new account data
 data ResetPasswordData = ResetPasswordData
@@ -401,7 +438,7 @@ passwordReset :: (Happstack m) =>
               -> m (Either PasswordError Text)
 passwordReset authenticateState passwordState passwordConfig =
   do method POST
-     (Just (Body body)) <- takeRequestBody =<< askRq
+     ~(Just (Body body)) <- takeRequestBody =<< askRq
      case Aeson.decode body of
        Nothing -> badRequest $ Left $ CoreError JSONDecodeFailed
        (Just (ResetPasswordData password passwordConfirm resetToken)) ->
@@ -450,7 +487,7 @@ decodeAndVerifyResetToken authenticateState token =
      case mUnverified of
        Nothing -> return Nothing
        (Just unverified) ->
-         case Map.lookup "reset-password" (unregisteredClaims (claims unverified)) of
+         case Map.lookup "reset-password" (unClaimsMap (unregisteredClaims (claims unverified))) of
            Nothing -> return Nothing
            (Just uv) ->
              case fromJSON uv of
@@ -460,7 +497,11 @@ decodeAndVerifyResetToken authenticateState token =
                     case mssecret of
                       Nothing -> return Nothing
                       (Just ssecret) ->
+#if MIN_VERSION_jwt(0,8,0)
+                        case verify (hmacSecret (_unSharedSecret ssecret)) unverified of
+#else
                         case verify (secret (_unSharedSecret ssecret)) unverified of
+#endif
                           Nothing -> return Nothing
                           (Just verified) ->
                             do now <- liftIO getPOSIXTime
